/*
 * eGovFrame Web Messager
 * Copyright The eGovFrame Open Community (http://open.egovframe.go.kr)).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author 이영지(슈퍼개발자K3)
 */
package egovframework.com.ext.msg.server;

import java.io.IOException;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import javax.websocket.EncodeException;
import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import egovframework.com.ext.msg.server.config.ChatServerAppConfig;
import egovframework.com.ext.msg.server.model.ChatMessage;
import egovframework.com.ext.msg.server.model.Message;
import egovframework.com.ext.msg.server.model.User;
import egovframework.com.ext.msg.server.model.UsersMessage;
import egovframework.com.ext.msg.server.model.decoder.MessageDecoder;
import egovframework.com.ext.msg.server.model.encoder.MessageEncoder;
import egovframework.rte.cmmn.util.DateUtil;

/**
* @Class Name : ChatServerEndPoint.java
* @Description : 두 명의 사용자가 대화할 때 접속 처리및 메시지처리 기능을 하는 WebSocket 서버클래스
* @Modification Information
*
*    수정일       수정자         수정내용
*    -------        -------     -------------------
*    2014. 11. 27.    이영지
*
*/
@ServerEndpoint(value = "/chat/{room}", encoders={MessageEncoder.class}, decoders={MessageDecoder.class}, configurator=ChatServerAppConfig.class)
public class ChatServerEndPoint {
	private static final Logger log = LogManager.getLogger();
	private Set<Session> chatroomUsers = Collections.synchronizedSet(new HashSet<Session>());

	/**
	 * Handshaking 함수
	 * @param userSession 사용자 session
	 */
	@OnOpen
	public void handleOpen(Session userSession, @PathParam("room") final String room) throws IOException, EncodeException {
		userSession.getUserProperties().put("room", room);
		chatroomUsers.add(userSession);
		log.debug("쳇팅 유저가 추가 되었습니다.("+room+")");
	}

	/**
	 * 메시지 전달 함수
	 * @param incomingMessage 들어오는 메시지
	 * @param userSession 사용자 session
	 * @param room room Id
	 * @throws IOException
	 * @throws EncodeException
	 */
	@OnMessage
	public void handleMessage(Message incomingMessage, Session userSession, @PathParam("room") final String room) throws IOException, EncodeException {

		ChatMessage incomingChatMessage = (ChatMessage)incomingMessage;
		ChatMessage outgoingChatMessage = new ChatMessage();

		String userId = (String) userSession.getUserProperties().get("userId");
		String userName = (String) userSession.getUserProperties().get("userName");
		try{
			if (userId == null) {
				log.debug("유저정보 준비");
				userId = incomingChatMessage.getUserId();
				userName = incomingChatMessage.getUserName();
				log.debug("유저정보 수신완료 ("+userId+")");
				log.debug("###########################유저정보 수신완료 ("+userId+")");
				log.debug("###########################유저정보 수신완료 ("+userName+")");
				if (userId != null) {
					userSession.getUserProperties().put("userId", userId);
					userSession.getUserProperties().put("userName", userName);
				}
	
				synchronized (chatroomUsers) {
					for (Session session : chatroomUsers){
						session.getBasicRemote().sendObject(new UsersMessage(getUsers()));
					}
				}
				log.debug("유저정보 송신완료");
			} else {
				log.info("#SEND#"+room+":"+userId+":"+incomingChatMessage.getMessage());
				outgoingChatMessage.setUserName(userName);
				outgoingChatMessage.setUserId(userId);
				outgoingChatMessage.setMessage(incomingChatMessage.getMessage());
	
				for (Session session : chatroomUsers){
					session.getBasicRemote().sendObject(outgoingChatMessage);
				}
			}
		}catch(Exception e){
			e.printStackTrace();
		}
	}


	//누군가가 접속 끊을때
	@OnClose
	public void handleClose(Session userSession, @PathParam("room") final String room) throws IOException, EncodeException{
		log.debug("쳇팅방에서 유저가 빠졌습니다.("+room+")");
		chatroomUsers.remove(userSession);
		
		UsersServerEndPoint.delChatRoomInfoFromUserSession(room, (String)userSession.getUserProperties().get("userId"));
		
		ChatMessage outgoingChatMessage = new ChatMessage();

		log.info("#EXIT#"+room+":"+(String)userSession.getUserProperties().get("userId")+" EXIT.");
		for (Session session : chatroomUsers){
			session.getBasicRemote().sendObject(new UsersMessage(getUsers()));
			outgoingChatMessage.setUserName("알림");
			outgoingChatMessage.setUserId("@ChatServer");
			outgoingChatMessage.setRoom(room);			
			outgoingChatMessage.setMessage((String)userSession.getUserProperties().get("userName")+"님이 대화방에서 떠났습니다.");
			session.getBasicRemote().sendObject(outgoingChatMessage);
		}
	}

	/**
	 * 사용자가 접속 끊기 전 호출되는 함수
	 * @param session
	 * @param throwable
	 * @param room
	 */
	@OnError
    public void handleError(Session session, Throwable throwable, @PathParam("room") final String room) {
        // Error handling
		log.info("ChatServerEndPoint (room: "+room+") occured Exception!");
		log.info("Exception : "+throwable.getMessage());
		log.error(throwable);
		throwable.printStackTrace();
    }

	/**
	 * 사용자 정보를 가져오는 함수
	 * @return
	 */
	private Set<User> getUsers() {
		HashSet<User> returnSet = new HashSet<User>();
		log.debug("사용자 정보를 가져옵니다.");
		for (Session session : chatroomUsers){
			if (session.getUserProperties().get("userId") != null) {
				User user = new User(session);
				returnSet.add(user);
			}
		}
		return returnSet;
	}

}
