/*
 * eGovFrame Web Messager
 * Copyright The eGovFrame Open Community (http://open.egovframe.go.kr)).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author 이영지(슈퍼개발자K3)
 */
package egovframework.com.ext.msg.server;

import java.io.IOException;
import java.io.StringReader;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.json.Json;
import javax.json.JsonArrayBuilder;
import javax.json.JsonObject;
import javax.json.JsonObjectBuilder;
import javax.websocket.EncodeException;
import javax.websocket.OnClose;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.ServerEndpoint;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import egovframework.com.ext.msg.server.model.ChatMessage;


/**
* @Class Name : UsersServerEndPoint.java
* @Description : 현재 가능한 대화사용자 리스트를 처리하는 WebSocket 서버클래스
* @Modification Information
*
*    수정일       수정자         수정내용
*    -------        -------     -------------------
*    2014. 11. 27.    이영지
*
*/
@ServerEndpoint(value = "/usersServerEndpoint"/* ,configurator=ServerAppConfig.class*/)
public class UsersServerEndPoint {
	private static final Logger log = LogManager.getLogger();
	private static Set<Session> connectedAllUsers = Collections.synchronizedSet(new HashSet<Session>());
	private static Map<String,Set<Session>> chatingRooms = new HashMap<String,Set<Session>>();

	//Spring bean과 연동하기 위해서는 ServerAppConfig를 configurator로 등록해주면 된다.
	/*
	 * @Resource(name="TestService") TestService testService;
	 */

	/**
	 * Handshaking 함수
	 * @param userSession 사용자 session
	 */
	@OnOpen
	public void handleOpen(Session userSession) {
		connectedAllUsers.add(userSession);
		log.debug("user added.. total user count: "+connectedAllUsers.size());
	}

	/**
	 * Message전달 함수
	 * @param message 메시지
	 * @param userSession 사용자 session
	 * @throws IOException
	 * @throws EncodeException
	 */
	@OnMessage
	public void handleMessage(String message, Session userSession) throws IOException, EncodeException {
		log.debug("메시지가 수신되었습니다.");
		log.debug(message);
		String userId = (String) userSession.getUserProperties().get("userId");
		String userName = (String) userSession.getUserProperties().get("userName");
		String deptId = (String) userSession.getUserProperties().get("deptId");

		JsonObject jsonObject = Json.createReader(new StringReader(message)).readObject();

		String connectionType = jsonObject.getString("connectionType");
		if ("firstConnection".equals(connectionType) && userId == null) {
			log.debug("firstConnection");
			// 맨 처음 접속 시,
			// 사용자의 이름을 가져옴
			userId = jsonObject.getString("userId");
			userName = jsonObject.getString("userName");
			deptId = jsonObject.getString("deptId");

			log.info(userName + " is entered.");
			//isExisted(userId);
			if (userId != null && !isExisted(userId) ) {
				userSession.getUserProperties().put("userId", userId);
				userSession.getUserProperties().put("userName", userName);
				userSession.getUserProperties().put("deptId", deptId);

				for (Session session : connectedAllUsers) {
					String anotherUserId = (String)session.getUserProperties().get("userId");
					/*
					 * 다른두곳에서 같은 아이디로 접속했다면?
					 */
					if(anotherUserId!=null && anotherUserId.equals(userId)){
						log.debug("전체사용자정보를 보냅니다."+anotherUserId);
						session.getBasicRemote().sendText(buildJsonUserData(getUsers()));
					}else{
						log.debug("한사람 사용자정보를 보냅니다."+anotherUserId);
						session.getBasicRemote().sendText(buildJsonUserData(userSession.getUserProperties()));
					}
						
				}
			} else {
				// username을 다시 입력하게하는 로직 넣기.
			}

		} else if ("chatConnection".equals(connectionType)) {
			// chatroomId로 또다른 webSocket url에 접근한다.
			// id generation으로 대체가능.
			String roomId = genRandom();
			
			// 다른 사용자와 대화하고자 시도할 때
			// 채팅룸 사용자 저장
			
			addChatRoomInfoToUserSession(roomId,userSession);
			
			Set<Session> chatroomMembers = new HashSet<Session>();
			chatroomMembers.add(userSession);
			
			// 선택한 사용자를 사용자들 안에서 찾기.
			String connectingUser = jsonObject.getString("connectingUser");
			String requestUser = jsonObject.getString("requestUser");
			log.info("#JOIN#"+roomId+":"+requestUser);

			
			if (connectingUser != null && !userId.equals(connectingUser)) {
				// 사용자들 중 선택한 유저와 연결
				for (Session session : connectedAllUsers) {
					if (connectingUser.equals(session.getUserProperties().get("userId"))) {
						// 선택한 사용자면 chatroomMember로 추가.
						
						addChatRoomInfoToUserSession(roomId,session);
						
						chatroomMembers.add(session);
						log.info("#JOIN#"+roomId+":"+connectingUser); 					
					}
				}

				// chatroomMembers에게 room입장하라는 신호 보내기
				for (Session session : chatroomMembers) {

					session.getBasicRemote().sendText(
						Json.createObjectBuilder()
						.add("enterChatId", roomId)
						.add("username", (String) session.getUserProperties().get("userName"))
						.add("userId", (String) session.getUserProperties().get("userId"))
						.add("requestUser", requestUser)
						.build().toString()
					);
				}
				
				chatingRooms.put(roomId, chatroomMembers);
				
			}
		} else if("existedChatConnection".equals(connectionType)){
			
			String roomId = jsonObject.getString("roomId");
			String connectingUser = jsonObject.getString("connectingUser");
			String requestUser = jsonObject.getString("requestUser");
			
			Set<Session> chatroomMembers = chatingRooms.get(roomId);
			
			if(chatroomMembers !=null && chatroomMembers.size()>0 && connectingUser !=null && !userId.equals(connectingUser)){
				for(Session session : connectedAllUsers){
					if(connectingUser.equals(session.getUserProperties().get("userId"))){
						
						addChatRoomInfoToUserSession(roomId,userSession);
						
						chatroomMembers.add(session);
						log.info("#JOIN#"+roomId+":"+connectingUser);
					}
				}
			}
			
			// chatroomMembers에게 room입장하라는 신호 보내기
			for (Session session : chatroomMembers) {
				if(connectingUser.equals(session.getUserProperties().get("userId")) && !isAlreadyJoined(roomId,session)){
					session.getBasicRemote().sendText(
						Json.createObjectBuilder()
						.add("enterChatId", roomId)
						.add("username", (String) session.getUserProperties().get("userName"))
						.add("userId", (String) session.getUserProperties().get("userId"))
						.add("requestUser", requestUser)
						.build().toString()
					);
				}else{
					session.getBasicRemote().sendText(
						Json.createObjectBuilder()
						.add("enterExistedChatId", roomId)
						.add("username", (String) session.getUserProperties().get("userName"))
						.add("userId", (String) session.getUserProperties().get("userId"))
						.add("requestUser", requestUser)
						.build().toString()
					);
				}
			}
			
		} else if ("groupChatConnection".equals(connectionType)) {
			
			// chatroomId로 또다른 webSocket url에 접근한다.
			// id generation으로 대체가능.
			String roomId = genRandom();
			
			// 다른 사용자와 대화하고자 시도할 때
			// 채팅룸 사용자 저장
			
			addChatRoomInfoToUserSession(roomId,userSession);
			
			Set<Session> chatroomMembers = new HashSet<Session>();
			chatroomMembers.add(userSession);
			
			// 선택한 사용자를 사용자들 안에서 찾기.
			String connectingUser[] = jsonObject.getString("connectingUser").split(",");
			
			String requestUser = jsonObject.getString("requestUser");
			log.info("#JOIN#"+roomId+":"+requestUser);
			for(int i=0; i<connectingUser.length; i++) {
				if (connectingUser[i] != null && !userId.equals(connectingUser[i])) {
					// 사용자들 중 선택한 유저와 연결
					for (Session session : connectedAllUsers) {
						if (connectingUser[i].equals(session.getUserProperties().get("userId"))) {
							// 선택한 사용자면 chatroomMember로 추가.
							addChatRoomInfoToUserSession(roomId,session);
							chatroomMembers.add(session);
							log.info("#JOIN#"+roomId+":"+connectingUser[i]);
						}
					}
					
				}
			}
			// chatroomMembers에게 room입장하라는 신호 보내기
			for (Session session : chatroomMembers) {
				session.getBasicRemote().sendText(
						Json.createObjectBuilder()
						.add("enterChatId", roomId)
						.add("username", (String) session.getUserProperties().get("userName"))
						.add("userId", (String) session.getUserProperties().get("userId"))
						.add("requestUser", requestUser)
						.build().toString()
						);
			}
			chatingRooms.put(roomId, chatroomMembers);
		} else if ("msgAlarmSend".equals(connectionType)){
			 
			
			String sendNm = jsonObject.getString("sendNm");
			String[] recvId = jsonObject.getString("recvId").split(",");
			
			
			System.out.println(sendNm);
			for(int i=0; i<recvId.length;i++){
				System.out.println(recvId[i]);
			}
			
			//쪽지 알람 보내기
			for (Session session : connectedAllUsers) {
				for(int i=0; i<recvId.length;i++) {
					if(recvId[i].equals(session.getUserProperties().get("userId"))) {					
					session.getBasicRemote().sendText(
							Json.createObjectBuilder()
							.add("msgNotify", "msgNotify")
							.add("recvId", recvId[i])											
							.add("sendNm", sendNm)
							.build().toString()
							);
					}
				}
				;
			}	
		}
	}

	/**
	 * 연결을 끊기 직전에 호출되는 함수
	 * @param userSession
	 * @throws IOException
	 * @throws EncodeException
	 */
	// 예외처리 필요!
	@OnClose
	public void handleClose(Session userSession) throws IOException, EncodeException {

		log.debug("@############ handleClose");
		String disconnectedUserId = (String) userSession.getUserProperties().get("userId");
		String disconnectedUserName = (String) userSession.getUserProperties().get("userName");
		connectedAllUsers.remove(userSession);
		log.debug(disconnectedUserName+"is disconnected, connectedAllUsers size :"+connectedAllUsers.size() );

		if (disconnectedUserId != null) {
			Json.createObjectBuilder().add("disconnectedUser", disconnectedUserId).build().toString();
			
			delAllChatRoomJoinded(userSession);
			
			for (Session session : connectedAllUsers) {
				session.getBasicRemote().sendText(Json.createObjectBuilder().add("disconnectedUser", disconnectedUserId).build().toString());
			}
		}
	}

	/**
	 * 연결되어있는 user정보를 가져오는 함수
	 * @return user set
	 */
	private Set<Map<String,Object>> getUsers() {
		HashSet<Map<String,Object>> returnSet = new HashSet<Map<String,Object>>();
		log.debug("getUsers user count:"+connectedAllUsers.size());
		for (Session session : connectedAllUsers) {
			if (session.getUserProperties().get("userId") != null) {
				returnSet.add(session.getUserProperties());
			};
		}
		return returnSet;
	}
	
	/**
	 * 방금 접속한 사람만 변환
	 * @param map
	 * @return
	 */
	private String buildJsonUserData(Map<String,Object> map){
		
		JsonObjectBuilder jsonObjectBuilder = Json.createObjectBuilder();
		jsonObjectBuilder.add("userId",(String)map.get("userId"));
		jsonObjectBuilder.add("userName",(String)map.get("userName"));
		jsonObjectBuilder.add("deptId",(String)map.get("deptId"));
		
		return Json.createObjectBuilder().add("oneUser", jsonObjectBuilder).build().toString();
		
	}

	/**
	 * 유저 정보가 담긴 Set<String>을 json으로 변환해주는 함수
	 * @param set
	 * @return jsondata
	 */
	private String buildJsonUserData(Set<Map<String,Object>> set) {

   		JsonArrayBuilder jsonArrayBuilder = Json.createArrayBuilder();

		for (Map<String,Object> user : set) {
			JsonObjectBuilder jsonObjectBuilder = Json.createObjectBuilder();
			jsonObjectBuilder.add("userId",(String)user.get("userId"));
			jsonObjectBuilder.add("userName",(String)user.get("userName"));
			jsonObjectBuilder.add("deptId",(String)user.get("deptId"));
			jsonArrayBuilder.add(jsonObjectBuilder);
		}
		
		return Json.createObjectBuilder().add("allUsers", jsonArrayBuilder).build().toString();
	}

	/**
	 * 동일한 userId을 가진 user session이 있는지 확인하는 함수
	 * @param userId 
	 * @return 존재여부
	 * @throws IOException 
	 */
	private boolean isExisted(String userId) throws IOException {
		// 이미 userId을 가진 session이 있는지 검사.
		for (Session existedUser : connectedAllUsers) {
			if (userId.equals(existedUser.getUserProperties().get("userId"))) {
				existedUser.close();
				//existedUser.getBasicRemote().sendText(Json.createObjectBuilder().add("disconnectedUser", existedUser.getUserProperties().get("userId").toString()).build().toString());
//				session.getBasicRemote().sendText(Json.createObjectBuilder().add("disconnectedUser", disconnectedUserId).build().toString());
				return false;
			}
		}
		return false;
	}

	/**
	 * chatroomId를 위한 랜덤값을 생성하는 함수
	 * @return chatroomId
	 */
	private String genRandom() {
		String chatroomId = "";
		for (int i = 0; i < 8; i++) {
			chatroomId += (char) ((Math.random() * 26) + 97);
		}
		return chatroomId;
	}
	
	private void addChatRoomInfoToUserSession(String roomId,Session userSession){
		@SuppressWarnings("unchecked")
		Set<String> joinChatRooms = (Set<String>)userSession.getUserProperties().get("joinChatRooms");
		if(joinChatRooms == null){
			joinChatRooms = new HashSet<String>();
		}
		joinChatRooms.add(roomId);
		//userSession.getUserProperties().put("joinChatRooms", joinChatRooms);
	}
	
	@SuppressWarnings("unchecked")
	public static void delChatRoomInfoFromUserSession(String roomId,String userId){
		log.debug("유저가 참여하고 있는 쳇팅룸 정보를 삭제합니다.("+roomId+","+userId+")");
		Session userSession = null;
		
		for(Session session:connectedAllUsers){
			if(userId.equals(session.getUserProperties().get("userId"))){
				userSession = session;
				break;
			}
		}
		
		if(userSession != null){
			@SuppressWarnings("unchecked")
			Set<String> joinChatRooms = (Set<String>)userSession.getUserProperties().get("joinChatRooms");
			if(joinChatRooms != null){
				log.debug("삭제전 참여쳇팅룸 정보갯수:"+joinChatRooms.size());
				joinChatRooms.remove(roomId);
				log.debug("삭제후 참여쳇팅룸 정보갯수:"+((Set<String>)userSession.getUserProperties().get("joinChatRooms")).size());
				//userSession.getUserProperties().put("joinChatRooms", joinChatRooms);
				
				Set<Session> chatingRoom = chatingRooms.get(roomId);
				if(chatingRoom!=null){
					
					chatingRoom.remove(userSession);
				}else{
					log.debug("서버 쳇팅룸 정보가 없습니다.");
				}
				
				checkChatRoom(roomId);
				
			}else{
				log.debug("유저가 참여하고 있는 쳇팅룸 정보를 찾을 수 없습니다.");
			}
		}else{
			log.debug("유저아이디를 접속정보에서 찾을 수 없습니다.");
		}
		
		log.debug("유저가 참여하고 있는 쳇팅룸 정보를 삭제했습니다.");
	}
	
	private void delAllChatRoomJoinded(Session userSession){
		@SuppressWarnings("unchecked")
		Set<String> joinChatRooms = (Set<String>)userSession.getUserProperties().get("joinChatRooms");
		if(joinChatRooms != null){
			for(String roomId : joinChatRooms){
				Set<Session> session = chatingRooms.get(roomId);
				session.remove(session);
				
				checkChatRoom(roomId);
			}
		}
	}
	
	private boolean isAlreadyJoined(String roomId,Session userSession){
		
		
		log.debug("이미 참여하고 있는지 확인 합니다.("+roomId+","+userSession.getUserProperties().get("userName")+")");
		
		boolean result = false;
		@SuppressWarnings("unchecked")
		Set<String> joinChatRooms = (Set<String>)userSession.getUserProperties().get("joinChatRooms");
		if(joinChatRooms == null) return false;
		
		result = joinChatRooms.contains(roomId); 
		
		log.debug("이미 참여하고 있는지 확인 하였습니다.("+result+"");
		return result;
	}
	
	private static void checkChatRoom(String roomId){
		Set<Session> chatingRoom = chatingRooms.get(roomId);
		if(chatingRoom.size()==0){
			log.debug("쳇팅룸 참여자가 없어 서버에서 정보를 삭제합니다.("+roomId+")");
			chatingRooms.remove(roomId);
		}
	}
	
	
	

}
